
//+------------------------------------------------------------------+
//|                              Expon MACD Avg calculation test.mq4 |
//|                                 Programmed by OneStepRemoved.com |
//|                                                                  |
//+------------------------------------------------------------------+

#define DECIMAL_CONVERSION 10
#define COMMENT_DIGITS 2

// defines for evaluating entry conditions
#define INITIAL_CROSS_BAR 2
#define LAST_BAR 1
#define THIS_BAR 0
#define NEGATIVE_VALUE -1

// defines for managing trade orders
#define RETRYCOUNT    10
#define RETRYDELAY    500

#define LONG          1
#define SHORT         -1
#define ALL           0

#define ORDER_COMMENT_S1 "EA Order Support Level 1"
#define ORDER_COMMENT_S2 "EA Order Support Level 2"
#define ORDER_COMMENT_S3 "EA Order Support Level 3"
#define ORDER_COMMENT_R1 "EA Order Resistance Level 1"
#define ORDER_COMMENT_R2 "EA Order Resistance Level 2"
#define ORDER_COMMENT_R3 "EA Order Resistance Level 3"

#define OPEN_ORDER_LIMIT   3



#property copyright ", Programmed by OneStepRemoved.com"
#property link      "www.onestepremoved.com"


extern   double   Lots                                =  0.1;
extern   int      NumPips                             =  3;
extern   int      Stop                                =  100;
extern   int      TakeProfit                          =  50;
extern   int      TrailStart                          =  15;
extern   int      TrailAmount                         =  1;
extern   int      MagicNumber;
extern   bool     WriteScreenshots                    =  true;
extern   int      MACD_EMAperiod                      =  2;
extern   int      FastLength                          =  12;
extern   int      SlowLength                          =  26;


bool bullishLvl1 = false;
bool bullishLvl2 = false;
bool bullishLvl3 = false;
bool bearishLvl1 = false;
bool bearishLvl2 = false;
bool bearishLvl3 = false;

datetime lastTradeTime;
int Slippage = 2;

string display = "";

double trailStart, trailAmount, numPipPts;

datetime currTimeStamp;
int minSLTPdstnc;

int init()
  {
      if (Digits == 3 || Digits == 5)   {
         Stop        *=    DECIMAL_CONVERSION;
         TakeProfit  *=    DECIMAL_CONVERSION;
         TrailStart  *=    DECIMAL_CONVERSION;
         TrailAmount *=    DECIMAL_CONVERSION;
         NumPips     *=    DECIMAL_CONVERSION;
      }
      
      trailStart     =     TrailStart  *  Point;
      trailAmount    =     TrailAmount *  Point;
      numPipPts      =     NumPips *  Point;
      
      minSLTPdstnc = MarketInfo(Symbol(), MODE_STOPLEVEL);

      lastTradeTime = Time[1];
      Print("Broker: " + AccountCompany());
      Comment("Loading... awaiting new tick");

   return(0);
  }

int deinit()  {   return(0);  }

//--------------------------------------------------------------------------

double calcEMAofMACD()
{
  int ix;

  // use an array with double the size of the MACD period 
  //  so more calculated value is sufficently "primed".
//  double MACDema[MACD_EMAperiod*2];  

  double MACDs[];
  ArrayResize (MACDs, MACD_EMAperiod);
  
  
  // Load up Exponential Moving Avg. of MACD array
//  for (ix = MACD_EMAperiod*2 - 1; ix >= 0; ix--)
  for (ix = 0; ix < MACD_EMAperiod; ix++)
  {
    MACDs[ix] = iMACD(NULL, 0, 12, 26, MACD_EMAperiod, PRICE_CLOSE, MODE_MAIN, THIS_BAR+ix);
  }
  
  // Calculate Exponential Moving Average on array
  double EMAofMACD = iMAOnArray(MACDs, 0, MACD_EMAperiod, 0, MODE_EMA, THIS_BAR);
  
  return (EMAofMACD);
  
}

//--------------------------------------------------------------------------

int start()
{


   // We are only running at first tick of each bar
   
   if (currTimeStamp != Time[0])
   {
      currTimeStamp = Time[0];
      
      // Display comments
      
      display = "bullishLvl1: " + ConvertBoolToStr(bullishLvl1) + "bullishLvl2: " + ConvertBoolToStr(bullishLvl2) + 
         "bullishLvl3: " + ConvertBoolToStr(bullishLvl3) + "  bearishLvl1: " + ConvertBoolToStr(bearishLvl1) + 
         "  bearishLvl2: " + ConvertBoolToStr(bearishLvl2) + "  bearishLvl3: " + ConvertBoolToStr(bearishLvl3);
      Comment(display);
   
   
      // Exit any trades in opposite direction
   
      if (bullishLvl1 || bullishLvl2 || bullishLvl3)
         ExitAll(SHORT);
     
      else if (bearishLvl1 || bearishLvl2 || bearishLvl3)
         ExitAll(LONG);


      // If needed, do trailing stops
        
      TrailStop(MagicNumber, trailStart, trailAmount);
   
   
      // Check for entries and if entry condition exists, do a trade
   
//      if (NoOpenPositionsExist(ORDER_COMMENT) && lastTradeTime != Time[THIS_BAR])
//      if (OpenPositionsCount(ORDER_COMMENT) < OPEN_ORDER_LIMIT)
//      {
      if (bullishLvl1)
      {
         if (DoTrade(LONG, Lots, Stop, TakeProfit, ORDER_COMMENT_S1))
            lastTradeTime = Time[THIS_BAR];
      }
      else if (bullishLvl2)
      {
         if (DoTrade(LONG, Lots, Stop, TakeProfit, ORDER_COMMENT_S2))
            lastTradeTime = Time[THIS_BAR];
      }
      else if (bullishLvl3)
      {
         if (DoTrade(LONG, Lots, Stop, TakeProfit, ORDER_COMMENT_S3))
            lastTradeTime = Time[THIS_BAR];
      }
      else if(bearishLvl1)
      {
         if (DoTrade(SHORT, Lots, Stop, TakeProfit, ORDER_COMMENT_R1))
            lastTradeTime = Time[THIS_BAR];
      }
      else if(bearishLvl2)
      {
         if (DoTrade(SHORT, Lots, Stop, TakeProfit, ORDER_COMMENT_R2))
            lastTradeTime = Time[THIS_BAR];
      }
      else if(bearishLvl3)
      {
         if (DoTrade(SHORT, Lots, Stop, TakeProfit, ORDER_COMMENT_R3))
            lastTradeTime = Time[THIS_BAR];
      }
//      }

      // Update Calculation parameters.     
      
      double prevHigh = iHigh(Symbol(), PERIOD_D1, LAST_BAR);
      double prevLow = iLow(Symbol(), PERIOD_D1, LAST_BAR);
      double prevClose = iClose(Symbol(), PERIOD_D1, LAST_BAR);
      double currHigh = High[LAST_BAR];
      double currLow = Low[LAST_BAR];
//      Print("  Previous High: " + prevHigh + "  Previous Low: " + prevLow + "  Previous Close: " + prevClose + 
//            "  Current High: " + currHigh + "  Current Low: " + currLow);
      
      double pivPoint = (prevHigh + prevLow + prevClose) / 3;  // pivot point
//      Print("  Pivot Point: " + pivPoint);      
      
      // resistances
      double resist1 = pivPoint * 2 - prevLow;
      double resist2 = pivPoint + prevHigh - prevLow;
      double resist3 = resist2 + prevHigh - prevLow;
//      Print("  Resistance 1: " + resist1 + "  Resistance 2: " + resist2 + "  Resistance 3: " + resist3); 
      
      // supports
      double supprt1 = pivPoint * 2 - prevHigh;
      double supprt2 = pivPoint - prevHigh + prevLow;
      double supprt3 = supprt2 - prevHigh + prevLow;
//      Print("  Support 1: " + supprt1 + "  Support 2: " + supprt2 + "  Support 3: " + supprt3); 
      
      // MACD and MACD average(EMA).
      double MACD = iMACD(NULL, 0, FastLength, SlowLength, MACD_EMAperiod, PRICE_CLOSE, MODE_MAIN, LAST_BAR);
      double MACDavg = calcEMAofMACD();
      
      // difference between MACD and MACD average.
      double MACDdiff = MACD - MACDavg;
//      Print("  MACD: " + MACD + "  MACD Expon. Avg.: " + MACDavg + "  Difference: " + MACDdiff); 
      
//      Print("  Num Pip Points: " + numPipPts);      
      
//      Print("  Low: " + Low[LAST_BAR] + "  High: " + High[LAST_BAR]);      
      
      //**DEBUG PRINTS & COMMENTS**  Start
/*      if (MACDdiff > 0)
      {
      if (Low[LAST_BAR] > (supprt1 - numPipPts) && Low[LAST_BAR] < (supprt1 + numPipPts))
        Print("bullish: low > support 1 - Num Pips AND < support 1 + Num Pips");
      if (Low[LAST_BAR] > (supprt2 - numPipPts) && Low[LAST_BAR] < (supprt2 + numPipPts))
        Print("bullish: low > support 2 - Num Pips AND < support 2 + Num Pips");
      if (Low[LAST_BAR] > (supprt3 - numPipPts) && Low[LAST_BAR] < (supprt3 + numPipPts))
        Print("bullish: low > support 3 - Num Pips AND < support 3 + Num Pips");
      }
      if (MACDdiff < 0)
      {
      if (High[LAST_BAR] > (resist1 - numPipPts) && High[LAST_BAR] < (resist1 + numPipPts))
        Print("bearish: High > resistance 1 - Num Pips AND < resistance 1 + Num Pips");
      if (High[LAST_BAR] > (resist2 - numPipPts) && High[LAST_BAR] < (resist2 + numPipPts))
        Print("bearish: High > resistance 2 - Num Pips AND < resistance 2 + Num Pips");
      if (High[LAST_BAR] > (resist3 - numPipPts) && High[LAST_BAR] < (resist3 + numPipPts))
        Print("bearish: High > resistance 1 - Num Pips AND < resistance 1 + Num Pips");
      }
*/      
      display = "\n\n  MACDdiff: " + MACDdiff + "  High: " + High[LAST_BAR];
      Comment(display);
      
      display = "\n  Low: " + Low[LAST_BAR] + "  S1 - Num Pips: " + (supprt1 - numPipPts) + "  S1 + Num Pips: " + (supprt1 + numPipPts)
         + "  S2 - Num Pips: " + (supprt2 - numPipPts) + "  S2 + Num Pips: " + (supprt2 + numPipPts);
      Comment(display);
      //**DEBUG PRINTS & COMMENTS**  End
      
      // Check for an entry condition.     

      if (MACDdiff > 0)
      {
         if (Low[LAST_BAR] > (supprt1 - numPipPts) && Low[LAST_BAR] < (supprt1 + numPipPts) && NoOpenPositionsExist(ORDER_COMMENT_S1))
         {
            bullishLvl1 = true;
            bullishLvl2 = false; bullishLvl3 = false;
            bearishLvl1 = false; bearishLvl2 = false; bearishLvl3 = false;
         }   
         else if (Low[LAST_BAR] > (supprt2 - numPipPts) && Low[LAST_BAR] < (supprt2 + numPipPts) && NoOpenPositionsExist(ORDER_COMMENT_S2)) 
         {
            bullishLvl2 = true;
            bullishLvl1 = false; bullishLvl3 = false;
            bearishLvl1 = false; bearishLvl2 = false; bearishLvl3 = false;
         }   
         else if (Low[LAST_BAR] > (supprt3 - numPipPts) && Low[LAST_BAR] < (supprt3 + numPipPts) && NoOpenPositionsExist(ORDER_COMMENT_S3)) 
         {
            bullishLvl3 = true;
            bullishLvl1 = false; bullishLvl2 = false;
            bearishLvl1 = false; bearishLvl2 = false; bearishLvl3 = false;
         }
         else
         {
            bullishLvl1 = false; bullishLvl2 = false; bullishLvl3 = false;
            bearishLvl1 = false; bearishLvl2 = false; bearishLvl3 = false;
         }   
      } 
      else if (MACDdiff < 0)
      {
         if (High[LAST_BAR] > (resist1 - numPipPts) && High[LAST_BAR] < (resist1 + numPipPts) && NoOpenPositionsExist(ORDER_COMMENT_R1))
         {
            bearishLvl1 = true;
            bearishLvl2 = false; bearishLvl3 = false;
            bullishLvl1 = false; bullishLvl2 = false; bullishLvl3 = false;
         }
         else if (High[LAST_BAR] > (resist2 - numPipPts) && High[LAST_BAR] < (resist2 + numPipPts) && NoOpenPositionsExist(ORDER_COMMENT_R2))
         {
            bearishLvl2 = true;
            bearishLvl1 = false; bearishLvl3 = false;
            bullishLvl1 = false; bullishLvl2 = false; bullishLvl3 = false;
         }
         else if (High[LAST_BAR] > (resist3 - numPipPts) && High[LAST_BAR] < (resist3 + numPipPts) && NoOpenPositionsExist(ORDER_COMMENT_R3))
         {
            bearishLvl3 = true;
            bearishLvl1 = false; bearishLvl2 = false;
            bullishLvl1 = false; bullishLvl2 = false; bullishLvl3 = false;
         }
         else
         {
            bullishLvl1 = false; bullishLvl2 = false; bullishLvl3 = false;
            bearishLvl1 = false; bearishLvl2 = false; bearishLvl3 = false;
         }   
      }
      else
      {
         bullishLvl1 = false; bullishLvl2 = false; bullishLvl3 = false;
         bearishLvl1 = false; bearishLvl2 = false; bearishLvl3 = false;
      }   

   }
   
   return(0);
}

//--------------------------------------------------------------------------

bool DoTrade(int dir, double volume, int stop, int take, string comment)  {

double sl, tp;

bool retVal = false;

   switch(dir)  {
      case LONG:
         if (stop != 0) { sl = (stop*Point); }
	 else { sl = 0; }
	 if (take != 0) { tp = (take*Point); }
	 else { tp = 0; }

	 retVal = OpenTrade(LONG, volume, sl, tp, comment);
	 break;

      case SHORT:
         if (stop != 0) { sl = (stop*Point); }
	 else { sl = 0; }
	 if (take != 0) { tp = (take*Point); }
	 else { tp = 0; }

	 retVal = OpenTrade(SHORT, volume, sl, tp, comment);
	 break;

   }

return(retVal);

}

bool OpenTrade(int dir, double volume, double stop, double take, string comment, int t = 0)  {
    int i, j, ticket, cmd;
    double prc, sl, tp, lots;
    string cmt;

    Print("OpenTrade("+dir+","+DoubleToStr(volume,3)+","+DoubleToStr(stop,Digits)+","+DoubleToStr(take,Digits)+","+t+")");

    lots = CheckLots(volume);

    for (i=0; i<RETRYCOUNT; i++) {
        for (j=0; (j<50) && IsTradeContextBusy(); j++)
            Sleep(100);
        RefreshRates();

        if (dir == LONG) {
            cmd = OP_BUY;
            prc = Ask;
            sl = stop;
            tp = take;
        }
        if (dir == SHORT) {
            cmd = OP_SELL;
            prc = Bid;
            sl = stop;
            tp = take;
        }
        Print("OpenTrade: prc="+DoubleToStr(prc,Digits)+" sl="+DoubleToStr(sl,Digits)+" tp="+DoubleToStr(tp,Digits));

        cmt = comment;
        if (t > 0)
            cmt = comment + "|" + t;

        ticket = OrderSend(Symbol(), cmd, lots, prc, Slippage, 0, 0, cmt, MagicNumber);
        if (ticket != -1 && ( sl > 0 || tp > 0) ) {
            Print("OpenTrade: opened ticket " + ticket);
            Screenshot("OpenTrade");
            
            OrderSelect( ticket, SELECT_BY_TICKET, MODE_TRADES);

            for (i=0; i<RETRYCOUNT; i++) {
                for (j=0; (j<50) && IsTradeContextBusy(); j++)
                    Sleep(100);
                RefreshRates();

                if( dir == LONG ) {
                  if ( OrderModify(ticket, 0, OrderOpenPrice()-sl, OrderOpenPrice()+tp, 0) ) {
                      Print("OpenTrade: SL/TP are set LONG");
                      Screenshot("OpenTrade_SLTP");
                      break;
                  }
                }
                
                if( dir == SHORT ) {
                  if ( OrderModify(ticket, 0, OrderOpenPrice()+sl, OrderOpenPrice()-tp, 0) ) {
                      Print("OpenTrade: SL/TP are set SHORT");
                      Screenshot("OpenTrade_SLTP");
                      break;
                  }                
                }

                Print("OpenTrade: error \'"+ErrorDescription(GetLastError())+"\' when setting SL/TP");
                Sleep(RETRYDELAY);
            }

            return (true);
        }
        else{ 
        
         if( ticket > 0 ) {
            Screenshot("OpenTrade_SLTP");
            return( true );
         }
        }
        Print("OpenTrade: error \'"+ErrorDescription(GetLastError())+"\' when entering with "+DoubleToStr(lots,3)+" @"+DoubleToStr(prc,Digits));
        Sleep(RETRYDELAY);
    }

    Print("OpenTrade: can\'t enter after "+RETRYCOUNT+" retries");
    return (false);
}

string ConvertBoolToStr(bool value)
{
   if (value) { return("True"); }
   else { return("False"); }
}  

double CheckLots(double lots)
{
    double lot, lotmin, lotmax, lotstep, margin;
    
    lotmin = MarketInfo(Symbol(), MODE_MINLOT);
    lotmax = MarketInfo(Symbol(), MODE_MAXLOT);
    lotstep = MarketInfo(Symbol(), MODE_LOTSTEP);
    margin = MarketInfo(Symbol(), MODE_MARGINREQUIRED);

    if (lots*margin > AccountFreeMargin())
        lots = AccountFreeMargin() / margin;

    lot = MathFloor(lots/lotstep + 0.5) * lotstep;

    if (lot < lotmin)
        lot = lotmin;
    if (lot > lotmax)
        lot = lotmax;

    return (lot);
}


void Screenshot(string moment_name)
{
    if ( WriteScreenshots )
        WindowScreenShot(WindowExpertName()+"_"+Symbol()+"_M"+Period()+"_"+
                         Year()+"-"+two_digits(Month())+"-"+two_digits(Day())+"_"+
                         two_digits(Hour())+"-"+two_digits(Minute())+"-"+two_digits(Seconds())+"_"+
                         moment_name+".gif", 1024, 768);
}

string two_digits(int i)
{
    if (i < 10)
        return ("0"+i);
    else
        return (""+i);
}

bool NoOpenPositionsExist(string theComment)
{
   int total = OrdersTotal();
  
   for(int i = 0; i < total; i++)
   {
      OrderSelect(i,SELECT_BY_POS,MODE_TRADES);
      if (OrderMagicNumber() == MagicNumber && OrderSymbol() == Symbol() && OrderComment() == theComment)
      { return (false); }
   }

return (true);
}

bool Exit(int ticket, int dir, double volume, color clr, int t = 0)  {
    int i, j, cmd;
    double prc, sl, tp, lots;
    string cmt;

    bool closed;

    Print("Exit("+dir+","+DoubleToStr(volume,3)+","+t+")");

    for (i=0; i<RETRYCOUNT; i++) {
        for (j=0; (j<50) && IsTradeContextBusy(); j++)
            Sleep(100);
        RefreshRates();

        if (dir == LONG) {
            prc = Bid;
        }
        if (dir == SHORT) {
            prc = Ask;
       }
        Print("Exit: prc="+DoubleToStr(prc,Digits));

        closed = OrderClose(ticket,volume,prc,Slippage,clr);
        if (closed) {
            Print("Trade closed");
            Screenshot("Exit");

            return (true);
        }

        Print("Exit: error \'"+ErrorDescription(GetLastError())+"\' when exiting with "+DoubleToStr(volume,3)+" @"+DoubleToStr(prc,Digits));
        Sleep(RETRYDELAY);
    }

    Print("Exit: can\'t enter after "+RETRYCOUNT+" retries");
    return (false);
}


string ErrorDescription(int error_code)
{
    string error_string;

    switch( error_code ) {
        case 0:
        case 1:   error_string="no error";                                                  break;
        case 2:   error_string="common error";                                              break;
        case 3:   error_string="invalid trade parameters";                                  break;
        case 4:   error_string="trade server is busy";                                      break;
        case 5:   error_string="old version of the client terminal";                        break;
        case 6:   error_string="no connection with trade server";                           break;
        case 7:   error_string="not enough rights";                                         break;
        case 8:   error_string="too frequent requests";                                     break;
        case 9:   error_string="malfunctional trade operation (never returned error)";      break;
        case 64:  error_string="account disabled";                                          break;
        case 65:  error_string="invalid account";                                           break;
        case 128: error_string="trade timeout";                                             break;
        case 129: error_string="invalid price";                                             break;
        case 130: error_string="invalid stops";                                             break;
        case 131: error_string="invalid trade volume";                                      break;
        case 132: error_string="market is closed";                                          break;
        case 133: error_string="trade is disabled";                                         break;
        case 134: error_string="not enough money";                                          break;
        case 135: error_string="price changed";                                             break;
        case 136: error_string="off quotes";                                                break;
        case 137: error_string="broker is busy (never returned error)";                     break;
        case 138: error_string="requote";                                                   break;
        case 139: error_string="order is locked";                                           break;
        case 140: error_string="long positions only allowed";                               break;
        case 141: error_string="too many requests";                                         break;
        case 145: error_string="modification denied because order too close to market";     break;
        case 146: error_string="trade context is busy";                                     break;
        case 147: error_string="expirations are denied by broker";                          break;
        case 148: error_string="amount of open and pending orders has reached the limit";   break;
        case 149: error_string="hedging is prohibited";                                     break;
        case 150: error_string="prohibited by FIFO rules";                                  break;
        case 4000: error_string="no error (never generated code)";                          break;
        case 4001: error_string="wrong function pointer";                                   break;
        case 4002: error_string="array index is out of range";                              break;
        case 4003: error_string="no memory for function call stack";                        break;
        case 4004: error_string="recursive stack overflow";                                 break;
        case 4005: error_string="not enough stack for parameter";                           break;
        case 4006: error_string="no memory for parameter string";                           break;
        case 4007: error_string="no memory for temp string";                                break;
        case 4008: error_string="not initialized string";                                   break;
        case 4009: error_string="not initialized string in array";                          break;
        case 4010: error_string="no memory for array\' string";                             break;
        case 4011: error_string="too long string";                                          break;
        case 4012: error_string="remainder from zero divide";                               break;
        case 4013: error_string="zero divide";                                              break;
        case 4014: error_string="unknown command";                                          break;
        case 4015: error_string="wrong jump (never generated error)";                       break;
        case 4016: error_string="not initialized array";                                    break;
        case 4017: error_string="dll calls are not allowed";                                break;
        case 4018: error_string="cannot load library";                                      break;
        case 4019: error_string="cannot call function";                                     break;
        case 4020: error_string="expert function calls are not allowed";                    break;
        case 4021: error_string="not enough memory for temp string returned from function"; break;
        case 4022: error_string="system is busy (never generated error)";                   break;
        case 4050: error_string="invalid function parameters count";                        break;
        case 4051: error_string="invalid function parameter value";                         break;
        case 4052: error_string="string function internal error";                           break;
        case 4053: error_string="some array error";                                         break;
        case 4054: error_string="incorrect series array using";                             break;
        case 4055: error_string="custom indicator error";                                   break;
        case 4056: error_string="arrays are incompatible";                                  break;
        case 4057: error_string="global variables processing error";                        break;
        case 4058: error_string="global variable not found";                                break;
        case 4059: error_string="function is not allowed in testing mode";                  break;
        case 4060: error_string="function is not confirmed";                                break;
        case 4061: error_string="send mail error";                                          break;
        case 4062: error_string="string parameter expected";                                break;
        case 4063: error_string="integer parameter expected";                               break;
        case 4064: error_string="double parameter expected";                                break;
        case 4065: error_string="array as parameter expected";                              break;
        case 4066: error_string="requested history data in update state";                   break;
        case 4099: error_string="end of file";                                              break;
        case 4100: error_string="some file error";                                          break;
        case 4101: error_string="wrong file name";                                          break;
        case 4102: error_string="too many opened files";                                    break;
        case 4103: error_string="cannot open file";                                         break;
        case 4104: error_string="incompatible access to a file";                            break;
        case 4105: error_string="no order selected";                                        break;
        case 4106: error_string="unknown symbol";                                           break;
        case 4107: error_string="invalid price parameter for trade function";               break;
        case 4108: error_string="invalid ticket";                                           break;
        case 4109: error_string="trade is not allowed in the expert properties";            break;
        case 4110: error_string="longs are not allowed in the expert properties";           break;
        case 4111: error_string="shorts are not allowed in the expert properties";          break;
        case 4200: error_string="object is already exist";                                  break;
        case 4201: error_string="unknown object property";                                  break;
        case 4202: error_string="object is not exist";                                      break;
        case 4203: error_string="unknown object type";                                      break;
        case 4204: error_string="no object name";                                           break;
        case 4205: error_string="object coordinates error";                                 break;
        case 4206: error_string="no specified subwindow";                                   break;
        default:   error_string="unknown error";
    }

    return(error_string);
}

void ExitAll(int direction) {

   for (int i = 0; i < OrdersTotal(); i++) {
      OrderSelect(i, SELECT_BY_POS, MODE_TRADES);
      
      if(OrderSymbol() == Symbol() && OrderMagicNumber() == MagicNumber) {
         if (OrderType() == OP_BUY && direction == LONG) { Exit(OrderTicket(), LONG, OrderLots(), Blue); }
         if (OrderType() == OP_SELL && direction == SHORT) { Exit( OrderTicket(), SHORT, OrderLots(), Red); }
      }
   }
}

bool TrailStop(int magic, double trlStart, double trlAmount) {

   double profitPips, increments, sl;
   double chgFromOpen;


   for(int i = 0; i < OrdersTotal(); i++)
   {
      if (OrderSelect(i,SELECT_BY_POS,MODE_TRADES))
      {
         if (OrderMagicNumber() == magic && OrderSymbol() == Symbol())
         { 
            RefreshRates();   
            
            if( OrderType() == OP_BUY ) {
               chgFromOpen = NormalizeDouble(Bid - OrderOpenPrice(), Digits);
   
               // move to break even
               if(chgFromOpen >= trlStart && chgFromOpen > (minSLTPdstnc * Point) &&
                   NormalizeDouble(OrderStopLoss(),Digits) < NormalizeDouble(OrderOpenPrice(), Digits) && trlStart != 0 ) 
               {
                  Print("Moving stop to breakeven on order " + OrderTicket() + ". Bid is " + DoubleToStr( Bid, Digits ) + ", Trail start is " + DoubleToStr( trlStart, Digits ) );
                  return( OrderModify( OrderTicket(), OrderOpenPrice(), OrderOpenPrice(), OrderTakeProfit(), 0, Blue) );
               }
         
               profitPips = Bid - ( trlStart + OrderOpenPrice() ) ;
               if( trlAmount != 0) 
                  increments = MathFloor( profitPips / trlAmount ); 
               else 
                  increments = 0;
         
         
               if ( increments >= 1 && Bid >= OrderOpenPrice() + trlStart + ( increments * trlAmount ) ) {
                  sl = NormalizeDouble( OrderOpenPrice() + ( (increments-1)  * trlAmount ), Digits);
            
                  display = "\n\nRequested stop loss: " + DoubleToStr( sl, Digits);
            
                  if( sl > NormalizeDouble(OrderStopLoss(), Digits) ) {
            
                     if( OrderModify( OrderTicket(), OrderOpenPrice(), sl, OrderTakeProfit(), 0, Blue ) ){
                        Print("Trailng stop updated. Total increments: " + DoubleToStr(increments, Digits) );
                        return(true);
                     }
                  }
               }
            }
      
            if( OrderType() == OP_SELL ) {
               chgFromOpen = NormalizeDouble(OrderOpenPrice() - Ask, Digits);
            
               // move to break even
               if( chgFromOpen >= trlStart && chgFromOpen > (minSLTPdstnc * Point) &&
                  NormalizeDouble(OrderStopLoss(),Digits) > NormalizeDouble(OrderOpenPrice(), Digits) && trlStart != 0 ) {
                  Print("Moving stop to breakeven on order " + OrderTicket() + ". Ask is " + DoubleToStr( Ask, Digits ) + ", Trail start is " + DoubleToStr( trlStart, Digits ) );
                  return( OrderModify( OrderTicket(), OrderOpenPrice(), OrderOpenPrice(), OrderTakeProfit(), 0, Red) );
               }
         
               profitPips = ( OrderOpenPrice()- trlStart ) - Ask ;
               if( trlAmount != 0) { increments = MathFloor( profitPips / trlAmount ); }
               else { increments = 0; }
         
               if ( increments >= 1 && Ask <= OrderOpenPrice() - trlStart - ( increments * trlAmount ) ) {
                  sl = NormalizeDouble(OrderOpenPrice() - ( (increments-1) * trlAmount ), Digits);
            
                  if( sl < NormalizeDouble(OrderStopLoss(), Digits) ) {
                     if( OrderModify( OrderTicket(), OrderOpenPrice(), sl, OrderTakeProfit(), 0, Red) ) {
                        Print("Trailng stop updated. Total increments: " + DoubleToStr(increments, Digits) );
                        return(true);
                     }
                  }
               }
            }
      
            if( IsStopped() ) { break; } 
         }          
      }
   }
   
   return( false );
}
